/**************************************************************************\
*								 3DGROUP.CPP							   *
*																		   *
*			 Data types and functions for 3D perspective drawing		   *
*								Polygon group							   *
*																		   *
*								Version 1.1a							   *
*																		   *
*							  by Stewart Gordon							   *
*							  July-August 1998							   *
\**************************************************************************/
#include <stdarg.h>
#include "3dgroup.h"

// Class group: group of 3D polygons

// Constructor: put polygons in group
group::group(int numitems, polygon*, ...)
	  : indexlen(0), index(NULL), deletepolys(FALSE) {
	va_list ap;
	va_start(ap, numitems);

	for(int pno = 0; pno < numitems; pno++) {
		polygons.add(va_arg(ap, polygon*));
	}
	va_end(ap);
}

// Destructor
group::~group() {
	delete[] index;
	if(deletepolys)
		ITERATE(polygons) {
			delete *polygons;
		}
}

// Render all polygons in the group, and sort the group
void group::render(const view& vw) {
	ITERATE(polygons) {
		(*polygons)->render(vw);
	}
	sort();
}

// Create an index of the polygons, sorted in stacking order according to
// the current viewpoint (most distant first)
/*void group::sort() {
	const int numitems = polygons.numitems();

	// allocate space for index if not already allocated
	if(numitems != indexlen) {
		delete[] index;
		index = new polygon*[numitems];
		indexlen = numitems;
	}

	// create the sorted index
	for(int pno = 0; pno < indexlen; pno++) {
		int indexpos;
		polygon *curpoly = polygons[pno];
		// make space in the index for a new item
		for(indexpos = pno - 1; indexpos >= 0; indexpos--) {
			if(index[indexpos]->relorder(*curpoly) == BEHIND) {
				index[indexpos + 1] = index[indexpos];
			}
			else {
				break;
			}
		}
		// put the new item in place
		index[indexpos + 1] = curpoly;
	}
}*/

// Draw the group in the current order and view
void group::draw(const view& vw) {
	if(indexlen != polygons.numitems())
		sort(vw);
	for(int pno = 0; pno < indexlen; pno++) {
		index[pno]->draw(vw);
	}
}

void group::sort(const view& vw) {
	int numitems = polygons.numitems();
	ITERATE(polygons) {
		if((*polygons)->scrp == NULL) numitems--;
	}

	// allocate space for index if not already allocated
	if(numitems != indexlen) {
		delete[] index;
		index = new polygon*[numitems];
		indexlen = numitems;
	}

	// create the sorted index
	int pno = 0;
	ITERATE(polygons) {
		if((*polygons)->scrp == NULL) continue;
		int indexpos;
		polygon *curpoly = *polygons;
		// make space in the index for a new item
		for(indexpos = pno - 1; indexpos >= 0; indexpos--) {
			if(index[indexpos]->relorder(*curpoly, TRUE, vw) == BEHIND) {
				index[indexpos + 1] = index[indexpos];
			}
			else {
				break;
			}
		}
		// put the new item in place
		index[indexpos + 1] = curpoly;
		pno++;
	}
}

