/**************************************************************************\
*								  3DPOLY.H								   *
*																		   *
*			 Data types and functions for 3D perspective drawing		   *
*								  Polygons								   *
*																		   *
*								Version 1.3b							   *
*																		   *
*							  by Stewart Gordon							   *
*							 July 1998-June 1999						   *
\**************************************************************************/
#ifndef __3DPOLY_H
#define __3DPOLY_H

#ifndef __3DVECTOR_H
#include "3dvector.h"
#endif

#ifndef __POLYBASE_H
#include "polybase.h"
#endif

class relpoly;
class scrpoly;

// Class scolour: colour of surface
class scolour {
public:
	rgb diffuse;
	rgb specular;

	scolour(float dred = 1, float dgreen = 1, float dblue = 1,
		float sred = 0, float sgreen = 0, float sblue = 0)
	  : diffuse(dred, dgreen, dblue), specular(sred, sgreen, sblue) {}
};

// Class polygon: polygon in 3D space
class polygon : public polybase<point> {

	void setdefaultattr() {
		filled = TRUE;
		fillpattern = 1;	// solid fill
		fillcolour = 0;		// black
		linestyle = 0;		// solid line
		linepattern = 0;	// no user-defined pattern set
		linethickness = 1;	// one-pixel line width

		//colour = rgb(.75, .75, .75);		// 75% diffuse
		//speccolour = rgb(.25, .25, .25);	// 25% specular

		// TEST CODE
		//colour = rgb(0, 0, 0);
		//speccolour = rgb(1, 0, 0);
	}

public:
	relpoly *relp;
	scrpoly *scrp;
	rtdpoly *rtdp;

	bool filled;
	int fillpattern, fillcolour;
	//rgb colour;
	//rgb speccolour;
	scolour *colour;
	int linestyle;
	unsigned linepattern;
	int linethickness;

	// Default constructor
	polygon() : polybase<point>(), relp(NULL), scrp(NULL), rtdp(NULL),
		  colour(NULL) {
		setdefaultattr();
	}

	// Constructor: Set number of points
	polygon(int NumPoints) : polybase<point>(NumPoints), relp(NULL),
		  scrp(NULL), rtdp(NULL), colour(NULL) {
		setdefaultattr();
	}

	polygon(int, int, int, int, ...);

	// Destructor
	~polygon() {
		//polybase<point>::~polybase();
		if(relp != NULL) delete relp;
		if(scrp != NULL) delete scrp;
		if(rtdp != NULL) delete rtdp;
	}

	point centre() const;
	bool behindeye(const view& = *curview) const;
	float distsq() const;

	vector normal() const;
	rgb appcolour(const view& = *curview) const;

	// Calculate the scalar product of the position vector of any point and
	// the normal vector.  This is the d in the standard equation forms
	// r.n=d or ax+by+cz=d
	float eqconst() const { return polypoints[0] % normal(); }

	orderrel relorder(const point&, const view& = *curview) const;
	orderrel relorder(const polygon&, bool = TRUE, const view& = *curview)
	  const;

	void render(const view& = *curview);
	void draw(const view& = *curview);
};

class relpoly : public polybase<relpoint> {
public:
	relpoly(const polygon*, const view& = *curview);

	void clip(float = 1);	// clip the polygon to a near z-plane

	relpoint centre() const;
	relvector normal() const;	// normal vector to polygon

	// Calculate the scalar product of the position vector of any point and
	// the normal vector.  This is the d in the standard equation forms
	// r.n=d or ax+by+cz=d
	float eqconst() const { return /*(relvector)*/polypoints[0] % normal(); }

	orderrel relorder(const relpoint&) const;
	orderrel relorder(const relpoly&, bool = TRUE) const;

	float bright(const relvector&) const;
	float specbright(const relvector&) const;
};

#endif
