/**************************************************************************\
*								 PALETTE.CPP 							   *
*																		   *
*		Data types and functions for dynamic palette colour selection	   *
*																		   *
*								 Version 1.3							   *
*																		   *
*							  by Stewart Gordon							   *
*							August-September 1998						   *
\**************************************************************************/
#include <graphics.h>
#include <stdio.h>
#include <miscids.h>
#include <conio.h>
#include "palette.h"
#define MAX_RGB	63

const int palette::palindex[16] = {
	0, 1, 2, 3, 4, 5, 20, 7, 56, 57, 58, 59, 60, 61, 62, 63
};

/*

// Set all components above the maximum to the maximum
void rgb::clipmax() {
	red = MIN(red, MAX_RGB);
	green = MIN(green, MAX_RGB);
	blue = MIN(blue, MAX_RGB);
}

// Set all components above the minimum to the minimum
void rgb::clipmin() {
	red = max(red, 0);
	green = max(green, 0);
	blue = max(blue, 0);
}

// Class rgb: red, green, blue colour value
// Add one RGB value to another
rgb rgb::operator+(rgb a) const {
	rgb v;
	v.red = red + a.red;
	v.green = green + a.green;
	v.blue = blue + a.blue;
	v.clipmax();
	return v;
}

// Subtract one RGB value from another
rgb rgb::operator-(rgb a) const {
	rgb v;
	v.red = red - a.red;
	v.green = green - a.green;
	v.blue = blue - a.blue;
	v.clipmin();
	return v;
}

// Multiply one RGB value by another
// Typically to calculate the light reflected off a coloured surface
rgb rgb::operator*(rgb a) const {
	rgb v;
	v.red = red * a.red / MAX_RGB;
	v.green = green * a.green / MAX_RGB;
	v.blue = blue * a.blue / MAX_RGB;
	return v;
}

// Multiply an RGB value by a scalar
rgb rgb::operator*(float f) const {
	rgb v;
	v.red = red * f;
	v.green = green * f;
	v.blue = blue * f;
	v.clipmax();
	return v;
}

// Saturation of colour
int rgb::sat() const {
	int v = val(), m = MIN(MIN(red, green), blue);

	if(v == 0) return 0;	// to prevent division by zero

	return (v - m) * 63 / v;
}

// Hue of colour
/*	  0	red
	 60	yellow
	120	green
	180	cyan
	240	blue
	300	magenta

int rgb::hue() const {
	int mx = val(), mn = MIN(MIN(red, green), blue);

	if(mx == mn) return 0;	// to prevent division by zero

	if(red == mx) {
		if(blue == mn) {	// red-yellow - 0-60
			return (green - mn) * 60 / (mx - mn);
		}	// if blue isn't minimum, green must be
			// magenta-red - 300-359
		return 360 - (blue - mn) * 60 / (mx - mn);
	}

	if(green == mx) {
		if(red == mn) {	// green-cyan - 120-180
			return 120 + (blue - mn) * 60 / (mx - mn);
		}	// if red isn't minimum, blue must be
			// yellow-green - 61-119
		return 120 - (red - mn) * 60 / (mx - mn);
	}

	// if neither red nor green is max, blue must be
	if(green == mn) {	// blue-magenta - 240-299
		return 240 + (red - mn) * 60 / (mx - mn);
	}	// if green isn't minimum, red must be
		// cyan-blue - 181-239
	return 240 - (green - mn) * 60 / (mx - mn);
}
*/


// Class palette: dynamic palette

// Find a colour in the palette, or add it if it's not already there
int palette::palcolour(int red, int green, int blue) {
	int newcol = -1;

	/* DECISION TABLE
		Status				UNUSED	REUSE	LAST	INUSE
		Is desired colour	Y	N	Y	N	Y	N	Y	N

		Use colour			N	N	Y	N	Y	N	Y	N
		Reuse palette entry	Y	Y	N	Y	N	N	N	N
	*/

	// find colour in palette
	for(int pidx = 0; pidx < pallen /*= palmax - palmin*/; pidx++) {
		/*if(pal[pidx].status == UNUSED) {
			if(newcol == -1) newcol = pidx;
		}*/
		//else {
			if(pal[pidx].status != UNUSED
				  && pal[pidx].red == red
				  && pal[pidx].green == green
				  && pal[pidx].blue == blue) {
				pal[pidx].status = INUSE;
				return pidx + palmin;
			}
			else if(pal[pidx].status <= REUSE) {
				if(newcol == -1) newcol = pidx;
			}
		//}
	}

	// colour not in palette - create a new colour
	if(newcol == -1) return -1;	// no room for new colour

	pidx = newcol;
	pal[pidx].red = red;
	pal[pidx].green = green;
	pal[pidx].blue = blue;
	pal[pidx].status = INUSE;
	setrgbpalette(palindex[pidx + palmin], red, green, blue);
	return pidx + palmin;
}

int palette::palcolour(rgb col) {
	/*// DEBUG CODE
	gotoxy(1, 2);
	printf("%f %f %f\n", col.red, col.green, col.blue);
	printf("%d %d %d\n", (int)(max(col.red, 1) * MAX_RGB),
	  (int)(max(col.green, 1) * MAX_RGB), (int)(max(col.blue, 1) * MAX_RGB));
	getch();*/

	return palcolour(MIN(col.red, 1) * MAX_RGB, MIN(col.green, 1) * MAX_RGB,
	  MIN(col.blue, 1) * MAX_RGB);
}

// Start the overlap between one palette and another
// (and end any previous overlap)
void palette::overlap() {
	for(int cno = 0; cno < pallen /*= palmax - palmin*/; cno++) {
		if(pal[cno].status == LAST) pal[cno].status = REUSE;
		if(pal[cno].status == INUSE) pal[cno].status = LAST;
	}
}

